@@ -1,11 +1,12 @@
 """Events for keystrokes and other input events"""
-import sys
-import encodings
 import codecs
+import encodings
+import itertools
+import sys
+from typing import Optional, List, Union
 
 from .termhelpers import Termmode
-
-from typing import Text, Optional, List, Union
+from .curtsieskeys import CURTSIES_NAMES as special_curtsies_names
 
 chr_byte = lambda i: chr(i).encode("latin-1")
 chr_uni = chr
@@ -23,56 +24,50 @@ for i in range(0x00, 0x80):
 for i in range(0x00, 0x1B):  # Overwrite the control keys with better labels
     CURTSIES_NAMES[chr_byte(i + 0x80)] = "<Meta-Ctrl-%s>" % chr(i + 0x40)
 
-from .curtsieskeys import CURTSIES_NAMES as special_curtsies_names
-
 CURTSIES_NAMES.update(special_curtsies_names)
 
-CURSES_NAMES = {}
-CURSES_NAMES[b"\x1bOP"] = "KEY_F(1)"
-CURSES_NAMES[b"\x1bOQ"] = "KEY_F(2)"
-CURSES_NAMES[b"\x1bOR"] = "KEY_F(3)"
-CURSES_NAMES[b"\x1bOS"] = "KEY_F(4)"
-CURSES_NAMES[b"\x1b[15~"] = "KEY_F(5)"
-CURSES_NAMES[b"\x1b[17~"] = "KEY_F(6)"
-CURSES_NAMES[b"\x1b[18~"] = "KEY_F(7)"
-CURSES_NAMES[b"\x1b[19~"] = "KEY_F(8)"
-CURSES_NAMES[b"\x1b[20~"] = "KEY_F(9)"
-CURSES_NAMES[b"\x1b[21~"] = "KEY_F(10)"
-CURSES_NAMES[b"\x1b[23~"] = "KEY_F(11)"
-CURSES_NAMES[b"\x1b[24~"] = "KEY_F(12)"
-
-# see bpython #626
-CURSES_NAMES[b"\x1b[11~"] = "KEY_F(1)"
-CURSES_NAMES[b"\x1b[12~"] = "KEY_F(2)"
-CURSES_NAMES[b"\x1b[13~"] = "KEY_F(3)"
-CURSES_NAMES[b"\x1b[14~"] = "KEY_F(4)"
-
-CURSES_NAMES[b"\x1b[A"] = "KEY_UP"
-CURSES_NAMES[b"\x1b[B"] = "KEY_DOWN"
-CURSES_NAMES[b"\x1b[C"] = "KEY_RIGHT"
-CURSES_NAMES[b"\x1b[D"] = "KEY_LEFT"
-CURSES_NAMES[b"\x1b[F"] = "KEY_END"  # https://github.com/bpython/bpython/issues/490
-CURSES_NAMES[b"\x1b[H"] = "KEY_HOME"  # https://github.com/bpython/bpython/issues/490
-CURSES_NAMES[b"\x08"] = "KEY_BACKSPACE"
-CURSES_NAMES[b"\x1b[Z"] = "KEY_BTAB"
-
-# see curtsies #78 - taken from https://github.com/jquast/blessed/blob/e9ad7b85dfcbbba49010ab8c13e3a5920d81b010/blessed/keyboard.py#L409
-# fmt: off
-CURSES_NAMES[b'\x1b[1~'] = 'KEY_FIND'         # find
-CURSES_NAMES[b'\x1b[2~'] = 'KEY_IC'           # insert (0)
-CURSES_NAMES[b'\x1b[3~'] = 'KEY_DC'           # delete (.), "Execute"
-CURSES_NAMES[b'\x1b[4~'] = 'KEY_SELECT'       # select
-CURSES_NAMES[b'\x1b[5~'] = 'KEY_PPAGE'        # pgup   (9)
-CURSES_NAMES[b'\x1b[6~'] = 'KEY_NPAGE'        # pgdown (3)
-CURSES_NAMES[b'\x1b[7~'] = 'KEY_HOME'         # home
-CURSES_NAMES[b'\x1b[8~'] = 'KEY_END'          # end
-CURSES_NAMES[b'\x1b[OA'] = 'KEY_UP'           # up     (8)
-CURSES_NAMES[b'\x1b[OB'] = 'KEY_DOWN'         # down   (2)
-CURSES_NAMES[b'\x1b[OC'] = 'KEY_RIGHT'        # right  (6)
-CURSES_NAMES[b'\x1b[OD'] = 'KEY_LEFT'         # left   (4)
-CURSES_NAMES[b'\x1b[OF'] = 'KEY_END'          # end    (1)
-CURSES_NAMES[b'\x1b[OH'] = 'KEY_HOME'         # home   (7)
-# fmt: on
+CURSES_NAMES = {
+    b"\x1bOP": "KEY_F(1)",
+    b"\x1bOQ": "KEY_F(2)",
+    b"\x1bOR": "KEY_F(3)",
+    b"\x1bOS": "KEY_F(4)",
+    b"\x1b[15~": "KEY_F(5)",
+    b"\x1b[17~": "KEY_F(6)",
+    b"\x1b[18~": "KEY_F(7)",
+    b"\x1b[19~": "KEY_F(8)",
+    b"\x1b[20~": "KEY_F(9)",
+    b"\x1b[21~": "KEY_F(10)",
+    b"\x1b[23~": "KEY_F(11)",
+    b"\x1b[24~": "KEY_F(12)",
+    # see bpython #626
+    b"\x1b[11~": "KEY_F(1)",
+    b"\x1b[12~": "KEY_F(2)",
+    b"\x1b[13~": "KEY_F(3)",
+    b"\x1b[14~": "KEY_F(4)",
+    b"\x1b[A": "KEY_UP",
+    b"\x1b[B": "KEY_DOWN",
+    b"\x1b[C": "KEY_RIGHT",
+    b"\x1b[D": "KEY_LEFT",
+    b"\x1b[F": "KEY_END",  # https://github.com/bpython/bpython/issues/490
+    b"\x1b[H": "KEY_HOME",  # https://github.com/bpython/bpython/issues/490
+    b"\x08": "KEY_BACKSPACE",
+    b"\x1b[Z": "KEY_BTAB",
+    # see curtsies #78 - taken from https://github.com/jquast/blessed/blob/e9ad7b85dfcbbba49010ab8c13e3a5920d81b010/blessed/keyboard.py#L409
+    b"\x1b[1~": "KEY_FIND",  # find
+    b"\x1b[2~": "KEY_IC",  # insert (0)
+    b"\x1b[3~": "KEY_DC",  # delete (.), "Execute"
+    b"\x1b[4~": "KEY_SELECT",  # select
+    b"\x1b[5~": "KEY_PPAGE",  # pgup   (9)
+    b"\x1b[6~": "KEY_NPAGE",  # pgdown (3)
+    b"\x1b[7~": "KEY_HOME",  # home
+    b"\x1b[8~": "KEY_END",  # end
+    b"\x1b[OA": "KEY_UP",  # up     (8)
+    b"\x1b[OB": "KEY_DOWN",  # down   (2)
+    b"\x1b[OC": "KEY_RIGHT",  # right  (6)
+    b"\x1b[OD": "KEY_LEFT",  # left   (4)
+    b"\x1b[OF": "KEY_END",  # end    (1)
+    b"\x1b[OH": "KEY_HOME",  # home   (7)
+}
 
 KEYMAP_PREFIXES = set()
 for table in (CURSES_NAMES, CURTSIES_NAMES):
@@ -82,7 +77,7 @@ for table in (CURSES_NAMES, CURTSIES_NAMES):
                 KEYMAP_PREFIXES.add(k[:i])
 
 MAX_KEYPRESS_SIZE = max(
-    len(seq) for seq in (list(CURSES_NAMES.keys()) + list(CURTSIES_NAMES.keys()))
+    len(seq) for seq in itertools.chain(CURSES_NAMES.keys(), CURTSIES_NAMES.keys())
 )
 
 
@@ -99,14 +94,14 @@ class ScheduledEvent(Event):
     Custom events that occur at a specific time in the future should
     be subclassed from ScheduledEvent."""
 
-    def __init__(self, when):
-        # type: (float) -> None
+    def __init__(self, when: float) -> None:
         self.when = when
 
 
 class WindowChangeEvent(Event):
-    def __init__(self, rows, columns, cursor_dy=None):
-        # type: (int, int, int) -> None
+    def __init__(
+        self, rows: int, columns: int, cursor_dy: Optional[int] = None
+    ) -> None:
         self.rows = rows
         self.columns = columns
         self.cursor_dy = cursor_dy
@@ -114,8 +109,7 @@ class WindowChangeEvent(Event):
     x = width = property(lambda self: self.columns)
     y = height = property(lambda self: self.rows)
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         return "<WindowChangeEvent (%d, %d)%s>" % (
             self.rows,
             self.columns,
@@ -123,21 +117,18 @@ class WindowChangeEvent(Event):
         )
 
     @property
-    def name(self):
-        # type: () -> str
+    def name(self) -> str:
         return "<WindowChangeEvent>"
 
 
 class SigIntEvent(Event):
     """Event signifying a SIGINT"""
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         return "<SigInt Event>"
 
     @property
-    def name(self):
-        # type: () -> str
+    def name(self) -> str:
         return repr(self)
 
 
@@ -147,22 +138,18 @@ class PasteEvent(Event):
     The events attribute contains a list of keypress event strings.
     """
 
-    def __init__(self):
-        # type: () -> None
-        self.events = []  # type: List[Union[Event, str]]
+    def __init__(self) -> None:
+        self.events: List[str] = []
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         return "<Paste Event with data: %r>" % self.events
 
     @property
-    def name(self):
-        # type: () -> str
+    def name(self) -> str:
         return repr(self)
 
 
-def decodable(seq, encoding):
-    # type: (bytes, str) -> bool
+def decodable(seq: bytes, encoding: str) -> bool:
     try:
         u = seq.decode(encoding)
     except UnicodeDecodeError:
@@ -171,8 +158,9 @@ def decodable(seq, encoding):
         return True
 
 
-def get_key(bytes_, encoding, keynames="curtsies", full=False):
-    # type: (List[bytes], str, str, bool) -> Optional[Text]
+def get_key(
+    bytes_: List[bytes], encoding: str, keynames: str = "curtsies", full: bool = False
+) -> Optional[str]:
     """Return key pressed from bytes_ or None
 
     Return a key name or None meaning it's an incomplete sequence of bytes
@@ -213,8 +201,7 @@ def get_key(bytes_, encoding, keynames="curtsies", full=False):
     if len(seq) > MAX_KEYPRESS_SIZE:
         raise ValueError("unable to decode bytes %r" % seq)
 
-    def key_name():
-        # type: () -> Union[Text]
+    def key_name() -> str:
         if keynames == "curses":
             # may not be here (and still not decodable) curses names incomplete
             if seq in CURSES_NAMES:
@@ -261,8 +248,7 @@ def get_key(bytes_, encoding, keynames="curtsies", full=False):
         assert False, "should have raised an unicode decode error"
 
 
-def could_be_unfinished_char(seq, encoding):
-    # type: (bytes, Text) -> bool
+def could_be_unfinished_char(seq: bytes, encoding: str) -> bool:
     """Whether seq bytes might create a char in encoding if more bytes were added"""
     if decodable(seq, encoding):
         return False  # any sensible encoding surely doesn't require lookahead (right?)
@@ -276,8 +262,7 @@ def could_be_unfinished_char(seq, encoding):
         return True  # We don't know, it could be
 
 
-def could_be_unfinished_utf8(seq):
-    # type: (bytes) -> bool
+def could_be_unfinished_utf8(seq: bytes) -> bool:
     # http://en.wikipedia.org/wiki/UTF-8#Description
 
     # fmt: off
@@ -291,8 +276,7 @@ def could_be_unfinished_utf8(seq):
     # fmt: on
 
 
-def pp_event(seq):
-    # type: (Text) -> Union[Text, bytes]
+def pp_event(seq: Union[Event, str]) -> Union[str, bytes]:
     """Returns pretty representation of an Event or keypress"""
 
     if isinstance(seq, Event):
@@ -301,7 +285,7 @@ def pp_event(seq):
     # Get the original sequence back if seq is a pretty name already
     rev_curses = {v: k for k, v in CURSES_NAMES.items()}
     rev_curtsies = {v: k for k, v in CURTSIES_NAMES.items()}
-    bytes_seq = None  # type: Optional[bytes]
+    bytes_seq: Optional[bytes] = None
     if seq in rev_curses:
         bytes_seq = rev_curses[seq]
     elif seq in rev_curtsies:
@@ -314,13 +298,11 @@ def pp_event(seq):
     return repr(seq).lstrip("u")[1:-1]
 
 
-def curtsies_name(seq):
-    # type: (bytes) -> Union[Text, bytes]
+def curtsies_name(seq: bytes) -> Union[str, bytes]:
     return CURTSIES_NAMES.get(seq, seq)
 
 
-def try_keys():
-    # type: () -> None
+def try_keys() -> None:
     print(
         "press a bunch of keys (not at the same time, but you can hit them pretty quickly)"
     )
@@ -330,8 +312,7 @@ def try_keys():
     import os
     from .termhelpers import Cbreak
 
-    def ask_what_they_pressed(seq, Normal):
-        # type: (bytes, Termmode) -> None
+    def ask_what_they_pressed(seq: bytes, Normal: Termmode) -> None:
         print("Unidentified character sequence!")
         with Normal:
             while True:
