@@ -1,47 +1,45 @@
 import locale
+import logging
 import os
-import signal
 import select
+import signal
 import sys
 import termios
 import threading
 import time
 import tty
 
-import logging
-
-logger = logging.getLogger(__name__)
-
-
 from .termhelpers import Nonblocking
 from . import events
 
-from typing import Callable, Type, TextIO, Optional, List, Union, Text, cast, Tuple, Any
+from typing import Callable, Type, TextIO, Optional, List, Union, cast, Tuple, Any
 from types import TracebackType, FrameType
 
+
+logger = logging.getLogger(__name__)
 READ_SIZE = 1024
 assert READ_SIZE >= events.MAX_KEYPRESS_SIZE
 # if a keypress could require more bytes than we read to be identified,
 # the paste logic that reads more data as needed might not work.
 
 
-def is_main_thread():
-    # type: () -> bool
+def is_main_thread() -> bool:
     return threading.current_thread() == threading.main_thread()
 
 
 class ReplacedSigIntHandler:
-    def __init__(self, handler):
-        # type: (Callable) -> None
+    def __init__(self, handler: Callable) -> None:
         self.handler = handler
 
-    def __enter__(self):
-        # type: () -> None
-        self.orig_sigint_handler = signal.getsignal(signal.SIGINT)
-        signal.signal(signal.SIGINT, self.handler)
+    def __enter__(self) -> None:
+        self.orig_sigint_handler = signal.signal(signal.SIGINT, self.handler)
 
-    def __exit__(self, type=None, value=None, traceback=None):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         signal.signal(signal.SIGINT, self.orig_sigint_handler)
 
 
@@ -50,14 +48,12 @@ class Input:
 
     def __init__(
         self,
-        in_stream=None,
-        keynames="curtsies",
-        paste_threshold=events.MAX_KEYPRESS_SIZE + 1,
-        sigint_event=False,
-        signint_callback_provider=None,
-        disable_terminal_start_stop=False,
-    ):
-        # type: (TextIO, str, int, bool, None, bool) -> None
+        in_stream: Optional[TextIO] = None,
+        keynames: str = "curtsies",
+        paste_threshold: Optional[int] = events.MAX_KEYPRESS_SIZE + 1,
+        sigint_event: bool = False,
+        disable_terminal_start_stop: bool = False,
+    ) -> None:
         """Returns an Input instance.
 
         Args:
@@ -76,29 +72,23 @@ class Input:
         if in_stream is None:
             in_stream = sys.__stdin__
         self.in_stream = in_stream
-        self.unprocessed_bytes = (
-            []
-        )  # type: List[bytes]  # leftover from stdin, unprocessed yet
+        self.unprocessed_bytes: List[bytes] = []  # leftover from stdin, unprocessed yet
         self.keynames = keynames
         self.paste_threshold = paste_threshold
         self.sigint_event = sigint_event
         self.disable_terminal_start_stop = disable_terminal_start_stop
-        self.sigints = []  # type: List[events.SigIntEvent]
+        self.sigints: List[events.SigIntEvent] = []
 
-        self.readers = []  # type: List[int]
-        self.queued_interrupting_events = []  # type: List[Union[events.Event, Text]]
-        self.queued_events = []  # type: List[events.Event]
-        self.queued_scheduled_events = (
-            []
-        )  # type: List[Tuple[float, events.ScheduledEvent]]
+        self.readers: List[int] = []
+        self.queued_interrupting_events: List[Union[events.Event, str]] = []
+        self.queued_events: List[Union[events.Event, None]] = []
+        self.queued_scheduled_events: List[Tuple[float, events.ScheduledEvent]] = []
 
     # prospective: this could be useful for an external select loop
-    def fileno(self):
-        # type: () -> int
+    def fileno(self) -> int:
         return self.in_stream.fileno()
 
-    def __enter__(self):
-        # type: () -> Input
+    def __enter__(self) -> "Input":
         self.original_stty = termios.tcgetattr(self.in_stream)
         tty.setcbreak(self.in_stream, termios.TCSANOW)
 
@@ -119,32 +109,44 @@ class Input:
         if self.sigint_event and is_main_thread():
             self.orig_sigint_handler = signal.getsignal(signal.SIGINT)
             signal.signal(signal.SIGINT, self.sigint_handler)
+
+        self.wakeup_read_fd, wfd = os.pipe()
+        os.set_blocking(wfd, False)
+        if sys.version_info[0] == 3 and 5 <= sys.version_info[1] < 7:
+            signal.set_wakeup_fd(wfd)
+        elif sys.version_info[0] == 3 and 7 <= sys.version_info[1]:
+            signal.set_wakeup_fd(wfd, warn_on_full_buffer=False)
+
         return self
 
-    def __exit__(self, type=None, value=None, traceback=None):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         if (
             self.sigint_event
             and is_main_thread()
             and self.orig_sigint_handler is not None
         ):
             signal.signal(signal.SIGINT, self.orig_sigint_handler)
+        if sys.version_info[0] == 3 and sys.version_info[1] > 4:
+            signal.set_wakeup_fd(-1)
         termios.tcsetattr(self.in_stream, termios.TCSANOW, self.original_stty)
 
-    def sigint_handler(self, signum, frame):
-        # type: (Union[signal.Signals, int], FrameType) -> None
+    def sigint_handler(
+        self, signum: Union[signal.Signals, int], frame: FrameType
+    ) -> None:
         self.sigints.append(events.SigIntEvent())
 
-    def __iter__(self):
-        # type: () -> Input
+    def __iter__(self) -> "Input":
         return self
 
-    def __next__(self):
-        # type: () -> Union[None, Text, events.Event]
+    def __next__(self) -> Union[None, str, events.Event]:
         return self.send(None)
 
-    def unget_bytes(self, string):
-        # type: (bytes) -> None
+    def unget_bytes(self, string: bytes) -> None:
         """Adds bytes to be internal buffer to be read
 
         This method is for reporting bytes from an in_stream read
@@ -152,8 +154,9 @@ class Input:
 
         self.unprocessed_bytes.extend(string[i : i + 1] for i in range(len(string)))
 
-    def _wait_for_read_ready_or_timeout(self, timeout):
-        # type: (Union[float, int, None]) -> Tuple[bool, Optional[Union[events.Event, Text]]]
+    def _wait_for_read_ready_or_timeout(
+        self, timeout: Union[float, int, None]
+    ) -> Tuple[bool, Optional[Union[events.Event, str]]]:
         """Returns tuple of whether stdin is ready to read and an event.
 
         If an event is returned, that event is more pressing than reading
@@ -165,13 +168,25 @@ class Input:
         while True:
             try:
                 (rs, _, _) = select.select(
-                    [self.in_stream.fileno()] + self.readers, [], [], remaining_timeout
+                    [
+                        self.in_stream.fileno(),
+                        self.wakeup_read_fd,
+                    ]
+                    + self.readers,
+                    [],
+                    [],
+                    remaining_timeout,
                 )
                 if not rs:
                     return False, None
                 r = rs[0]  # if there's more than one, get it in the next loop
                 if r == self.in_stream.fileno():
                     return True, None
+                elif r == self.wakeup_read_fd:
+                    # In Python >=3.5 select won't raise this signal handler
+                    signal_number = ord(os.read(r, 1))
+                    if signal_number == signal.SIGINT:
+                        raise InterruptedError()
                 else:
                     os.read(r, 1024)
                     if self.queued_interrupting_events:
@@ -188,8 +203,9 @@ class Input:
                 if remaining_timeout is not None:
                     remaining_timeout = max(remaining_timeout - (time.time() - t0), 0)
 
-    def send(self, timeout=None):
-        # type: (Union[float, int, None]) -> Union[None, Text, events.Event]
+    def send(
+        self, timeout: Optional[Union[float, None]] = None
+    ) -> Union[None, str, events.Event]:
         """Returns an event or None if no events occur before timeout."""
         if self.sigint_event and is_main_thread():
             with ReplacedSigIntHandler(self.sigint_handler):
@@ -197,10 +213,8 @@ class Input:
         else:
             return self._send(timeout)
 
-    def _send(self, timeout):
-        # type: (Union[float, int, None]) -> Union[None, Text, events.Event]
-        def find_key():
-            # type: () -> Optional[Text]
+    def _send(self, timeout: Union[float, int, None]) -> Union[None, str, events.Event]:
+        def find_key() -> Optional[str]:
             """Returns keypress identified by adding unprocessed bytes or None"""
             current_bytes = []
             while self.unprocessed_bytes:
@@ -212,7 +226,6 @@ class Input:
                     full=len(self.unprocessed_bytes) == 0,
                 )
                 if e is not None:
-                    current_bytes = []
                     return e
             if current_bytes:  # incomplete keys shouldn't happen
                 raise ValueError("Couldn't identify key sequence: %r" % current_bytes)
@@ -226,7 +239,7 @@ class Input:
             return self.queued_interrupting_events.pop(0)
 
         if self.queued_scheduled_events:
-            self.queued_scheduled_events.sort()  # TODO use a data structure that inserts sorted
+            self.queued_scheduled_events.sort()
             when, _ = self.queued_scheduled_events[0]
             if when < time.time():
                 logger.debug(
@@ -287,8 +300,7 @@ class Input:
             assert e is not None
             return e
 
-    def _nonblocking_read(self):
-        # type: () -> int
+    def _nonblocking_read(self) -> int:
         """Returns the number of characters read and adds them to self.unprocessed_bytes"""
         with Nonblocking(self.in_stream):
             try:
@@ -301,34 +313,35 @@ class Input:
             else:
                 return 0
 
-    def event_trigger(self, event_type):
-        # type: (Type[events.Event]) -> Callable
+    def event_trigger(
+        self, event_type: Union[Type[events.Event], Callable[..., None]]
+    ) -> Callable[..., None]:
         """Returns a callback that creates events.
 
         Returned callback function will add an event of type event_type
         to a queue which will be checked the next time an event is requested."""
 
-        def callback(**kwargs):
-            # type: (**Any) -> None
+        def callback(**kwargs: Any) -> None:
             self.queued_events.append(event_type(**kwargs))  # type: ignore
 
         return callback
 
-    def scheduled_event_trigger(self, event_type):
-        # type: (Type[events.ScheduledEvent]) -> Callable
+    def scheduled_event_trigger(
+        self, event_type: Type[events.ScheduledEvent]
+    ) -> Callable[[float], None]:
         """Returns a callback that schedules events for the future.
 
         Returned callback function will add an event of type event_type
         to a queue which will be checked the next time an event is requested."""
 
-        def callback(when):
-            # type: (float) -> None
+        def callback(when: float) -> None:
             self.queued_scheduled_events.append((when, event_type(when=when)))
 
         return callback
 
-    def threadsafe_event_trigger(self, event_type):
-        # type: (Type[events.Event]) -> Callable
+    def threadsafe_event_trigger(
+        self, event_type: Union[Type[events.Event], Callable[..., None]]
+    ) -> Callable[..., None]:
         """Returns a callback to creates events, interrupting current event requests.
 
         Returned callback function will create an event of type event_type
@@ -338,8 +351,7 @@ class Input:
         readfd, writefd = os.pipe()
         self.readers.append(readfd)
 
-        def callback(**kwargs):
-            # type: (**Any) -> None
+        def callback(**kwargs: Any) -> None:
             # TODO use a threadsafe queue for this
             self.queued_interrupting_events.append(event_type(**kwargs))  # type: ignore
             logger.debug(
@@ -350,13 +362,11 @@ class Input:
         return callback
 
 
-def getpreferredencoding():
-    # type: () -> str
+def getpreferredencoding() -> str:
     return locale.getpreferredencoding() or sys.getdefaultencoding()
 
 
-def main():
-    # type: () -> None
+def main() -> None:
     with Input() as input_generator:
         print(repr(input_generator.send(2)))
         print(repr(input_generator.send(1)))
