@@ -4,16 +4,12 @@
 
 from typing import (
     Optional,
-    Text,
     IO,
     Dict,
-    Generic,
     TypeVar,
     Type,
     Tuple,
     Callable,
-    Any,
-    ByteString,
     cast,
     TextIO,
     Union,
@@ -21,7 +17,6 @@ from typing import (
 )
 from types import TracebackType
 
-import locale
 import logging
 import re
 import sys
@@ -42,82 +37,79 @@ T = TypeVar("T", bound="BaseWindow")
 
 
 class BaseWindow:
-    def __init__(self, out_stream=None, hide_cursor=True):
-        # type: (IO, bool) -> None
+    def __init__(
+        self, out_stream: Optional[IO] = None, hide_cursor: bool = True
+    ) -> None:
         logger.debug("-------initializing Window object %r------" % self)
         if out_stream is None:
             out_stream = sys.__stdout__
         self.t = blessings.Terminal(stream=out_stream, force_styling=True)
         self.out_stream = out_stream
         self.hide_cursor = hide_cursor
-        self._last_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
-        self._last_rendered_width = None  # type: Optional[int]
-        self._last_rendered_height = None  # type: Optional[int]
+        self._last_lines_by_row: Dict[int, Optional[FmtStr]] = {}
+        self._last_rendered_width: Optional[int] = None
+        self._last_rendered_height: Optional[int] = None
 
-    def scroll_down(self):
-        # type: () -> None
+    def scroll_down(self) -> None:
         logger.debug("sending scroll down message w/ cursor on bottom line")
 
         # since scroll-down only moves the screen if cursor is at bottom
         with self.t.location(x=0, y=1000000):
             self.write(SCROLL_DOWN)  # TODO will blessings do this?
 
-    def write(self, msg):
-        # type: (Text) -> None
+    def write(self, msg: str) -> None:
         self.out_stream.write(msg)
         self.out_stream.flush()
 
-    def __enter__(self):
-        # type: (T) -> T
+    def __enter__(self: T) -> T:
         logger.debug("running BaseWindow.__enter__")
         if self.hide_cursor:
             self.write(self.t.hide_cursor)
         return self
 
-    def __exit__(self, type, value, traceback):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         logger.debug("running BaseWindow.__exit__")
         if self.hide_cursor:
             self.write(self.t.normal_cursor)
 
-    def on_terminal_size_change(self, height, width):
-        # type: (int, int) -> None
+    def on_terminal_size_change(self, height: int, width: int) -> None:
         # Changing the terminal size breaks the cache, because it
         # is unknown how the window size change affected scrolling / the cursor
         self._last_lines_by_row = {}
         self._last_rendered_width = width
         self._last_rendered_height = height
 
-    def render_to_terminal(self, array, cursor_pos=(0, 0)):
-        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> Optional[int]
+    def render_to_terminal(
+        self, array: Union[FSArray, List[FmtStr]], cursor_pos: Tuple[int, int] = (0, 0)
+    ) -> Optional[int]:
         raise NotImplementedError
 
-    def get_term_hw(self):
-        # type: () -> Tuple[int, int]
+    def get_term_hw(self) -> Tuple[int, int]:
         """Returns current terminal height and width"""
         return self.t.height, self.t.width
 
     @property
-    def width(self):
-        # type: () -> int
+    def width(self) -> int:
         "The current width of the terminal window"
         return self.t.width
 
     @property
-    def height(self):
-        # type: () -> int
+    def height(self) -> int:
         "The current width of the terminal window"
         return self.t.height
 
-    def array_from_text(self, msg):
-        # type: (Text) -> FSArray
+    def array_from_text(self, msg: str) -> FSArray:
         """Returns a FSArray of the size of the window containing msg"""
         rows, columns = self.t.height, self.t.width
         return self.array_from_text_rc(msg, rows, columns)
 
     @classmethod
-    def array_from_text_rc(cls, msg, rows, columns):
-        # type: (Text, int, int) -> FSArray
+    def array_from_text_rc(cls, msg: str, rows: int, columns: int) -> FSArray:
         arr = FSArray(0, columns)
         i = 0
         for c in msg:
@@ -130,23 +122,9 @@ class BaseWindow:
             i += 1
         return arr
 
-    def fmtstr_to_stdout_xform(self):
-        # type: () -> Callable[[FmtStr], Text]
-        if sys.version_info[0] == 2:
-            if hasattr(self.out_stream, "encoding"):
-                encoding = self.out_stream.encoding
-            else:
-                encoding = locale.getpreferredencoding()
-
-            def for_stdout(s):
-                # type: (FmtStr) -> Text
-                return unicode(s).encode(encoding, "replace")
-
-        else:
-
-            def for_stdout(s):
-                # type: (FmtStr) -> Text
-                return str(s)
+    def fmtstr_to_stdout_xform(self) -> Callable[[FmtStr], str]:
+        def for_stdout(s: FmtStr) -> str:
+            return str(s)
 
         return for_stdout
 
@@ -167,29 +145,34 @@ class FullscreenWindow(BaseWindow):
         its out_stream; cached writes will be inaccurate.
     """
 
-    def __init__(self, out_stream=None, hide_cursor=True):
-        # type: (IO, bool) -> None
+    def __init__(
+        self, out_stream: Optional[IO] = None, hide_cursor: bool = True
+    ) -> None:
         """Constructs a FullscreenWindow
 
         Args:
             out_stream (file): Defaults to sys.__stdout__
             hide_cursor (bool): Hides cursor while in context
         """
-        BaseWindow.__init__(self, out_stream=out_stream, hide_cursor=hide_cursor)
+        super().__init__(out_stream=out_stream, hide_cursor=hide_cursor)
         self.fullscreen_ctx = self.t.fullscreen()
 
-    def __enter__(self):
-        # type: () -> FullscreenWindow
+    def __enter__(self) -> "FullscreenWindow":
         self.fullscreen_ctx.__enter__()
-        return BaseWindow.__enter__(self)
+        return super().__enter__()
 
-    def __exit__(self, type, value, traceback):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         self.fullscreen_ctx.__exit__(type, value, traceback)
-        BaseWindow.__exit__(self, type, value, traceback)
+        super().__exit__(type, value, traceback)
 
-    def render_to_terminal(self, array, cursor_pos=(0, 0)):
-        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> None
+    def render_to_terminal(
+        self, array: Union[FSArray, List[FmtStr]], cursor_pos: Tuple[int, int] = (0, 0)
+    ) -> None:
         """Renders array to terminal and places (0-indexed) cursor
 
         Args:
@@ -215,7 +198,7 @@ class FullscreenWindow(BaseWindow):
         if height != self._last_rendered_height or width != self._last_rendered_width:
             self.on_terminal_size_change(height, width)
 
-        current_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
+        current_lines_by_row: Dict[int, Optional[FmtStr]] = {}
         rows = list(range(height))
         rows_for_use = rows[: len(array)]
         rest_of_rows = rows[len(array) :]
@@ -264,13 +247,12 @@ class CursorAwareWindow(BaseWindow):
 
     def __init__(
         self,
-        out_stream=None,
-        in_stream=None,
-        keep_last_line=False,
-        hide_cursor=True,
-        extra_bytes_callback=None,
+        out_stream: Optional[IO] = None,
+        in_stream: Optional[IO] = None,
+        keep_last_line: bool = False,
+        hide_cursor: bool = True,
+        extra_bytes_callback: Optional[Callable[[bytes], None]] = None,
     ):
-        # type: (IO, IO, bool, bool, Callable[[ByteString], None]) -> None
         """Constructs a CursorAwareWindow
 
         Args:
@@ -283,12 +265,12 @@ class CursorAwareWindow(BaseWindow):
                 bytes inadvertently read in get_cursor_position(). If not
                 provided, a ValueError will be raised when this occurs.
         """
-        BaseWindow.__init__(self, out_stream=out_stream, hide_cursor=hide_cursor)
+        super().__init__(out_stream=out_stream, hide_cursor=hide_cursor)
         if in_stream is None:
             in_stream = sys.__stdin__
         self.in_stream = in_stream
-        self._last_cursor_column = None  # type: Optional[int]
-        self._last_cursor_row = None  # type: Optional[int]
+        self._last_cursor_column: Optional[int] = None
+        self._last_cursor_row: Optional[int] = None
         self.keep_last_line = keep_last_line
         self.cbreak = Cbreak(self.in_stream)
         self.extra_bytes_callback = extra_bytes_callback
@@ -299,16 +281,19 @@ class CursorAwareWindow(BaseWindow):
         # in the cursor query code of cursor diff
         self.in_get_cursor_diff = False
 
-    def __enter__(self):
-        # type: () -> CursorAwareWindow
+    def __enter__(self) -> "CursorAwareWindow":
         self.cbreak.__enter__()
         self.top_usable_row, _ = self.get_cursor_position()
         self._orig_top_usable_row = self.top_usable_row
         logger.debug("initial top_usable_row: %d" % self.top_usable_row)
-        return BaseWindow.__enter__(self)
+        return super().__enter__()
 
-    def __exit__(self, type, value, traceback):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         if self.keep_last_line:
             # just moves cursor down if not on last line
             self.write(SCROLL_DOWN)
@@ -317,10 +302,9 @@ class CursorAwareWindow(BaseWindow):
         self.write(self.t.clear_eos)
         self.write(self.t.clear_eol)
         self.cbreak.__exit__(type, value, traceback)
-        BaseWindow.__exit__(self, type, value, traceback)
+        super().__exit__(type, value, traceback)
 
-    def get_cursor_position(self):
-        # type: () -> Tuple[int, int]
+    def get_cursor_position(self) -> Tuple[int, int]:
         """Returns the terminal (row, column) of the cursor
 
         0-indexed, like blessings cursor positions"""
@@ -330,8 +314,7 @@ class CursorAwareWindow(BaseWindow):
         query_cursor_position = "\x1b[6n"
         self.write(query_cursor_position)
 
-        def retrying_read():
-            # type: () -> str
+        def retrying_read() -> str:
             while True:
                 try:
                     c = in_stream.read(1)
@@ -365,7 +348,7 @@ class CursorAwareWindow(BaseWindow):
                 col = int(m.groupdict()["column"])
                 extra = m.groupdict()["extra"]
                 if extra:
-                    if self.extra_bytes_callback:
+                    if self.extra_bytes_callback is not None:
                         self.extra_bytes_callback(
                             # TODO how do we know that this works?
                             extra.encode(cast(TextIO, in_stream).encoding)
@@ -382,8 +365,7 @@ class CursorAwareWindow(BaseWindow):
                         )
                 return (row - 1, col - 1)
 
-    def get_cursor_vertical_diff(self):
-        # type: () -> int
+    def get_cursor_vertical_diff(self) -> int:
         """Returns the how far down the cursor moved since last render.
 
         Note:
@@ -412,8 +394,7 @@ class CursorAwareWindow(BaseWindow):
             if not self.another_sigwinch:
                 return cursor_dy
 
-    def _get_cursor_vertical_diff_once(self):
-        # type: () -> int
+    def _get_cursor_vertical_diff_once(self) -> int:
         """Returns the how far down the cursor moved."""
         old_top_usable_row = self.top_usable_row
         row, col = self.get_cursor_position()
@@ -437,8 +418,9 @@ class CursorAwareWindow(BaseWindow):
         self._last_cursor_row = row
         return cursor_dy
 
-    def render_to_terminal(self, array, cursor_pos=(0, 0)):
-        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> int
+    def render_to_terminal(
+        self, array: Union[FSArray, List[FmtStr]], cursor_pos: Tuple[int, int] = (0, 0)
+    ) -> int:
         """Renders array to terminal, returns the number of lines scrolled offscreen
 
         Returns:
@@ -468,7 +450,7 @@ class CursorAwareWindow(BaseWindow):
         if height != self._last_rendered_height or width != self._last_rendered_width:
             self.on_terminal_size_change(height, width)
 
-        current_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
+        current_lines_by_row: Dict[int, Optional[FmtStr]] = {}
         rows_for_use = list(range(self.top_usable_row, height))
 
         # rows which we have content for and don't require scrolling
@@ -523,8 +505,7 @@ class CursorAwareWindow(BaseWindow):
         return offscreen_scrolls
 
 
-def demo():
-    # type: () -> None
+def demo() -> None:
     handler = logging.FileHandler(filename="display.log")
     logging.getLogger(__name__).setLevel(logging.DEBUG)
     logging.getLogger(__name__).addHandler(handler)
@@ -534,13 +515,13 @@ def demo():
         with input.Input(sys.stdin) as input_generator:
             rows, columns = w.t.height, w.t.width
             for c in input_generator:
-                assert isinstance(c, Text)
+                assert isinstance(c, str)
                 if c == "":
                     sys.exit()  # same as raise SystemExit()
                 elif c == "h":
-                    a = w.array_from_text(
+                    a: Union[List[FmtStr], FSArray] = w.array_from_text(
                         "a for small array"
-                    )  # type: Union[List[FmtStr], FSArray]
+                    )
                 elif c == "a":
                     a = [fmtstr(c * columns) for _ in range(rows)]
                 elif c == "s":
@@ -564,8 +545,7 @@ def demo():
                 w.render_to_terminal(a)
 
 
-def main():
-    # type: () -> None
+def main() -> None:
     handler = logging.FileHandler(filename="display.log")
     logging.getLogger(__name__).setLevel(logging.DEBUG)
     logging.getLogger(__name__).addHandler(handler)
