@@ -1,4 +1,3 @@
-from __future__ import unicode_literals
 """A more realtime netcat"""
 import sys
 import select
@@ -8,7 +7,7 @@ from curtsies import FullscreenWindow, Input, FSArray
 from curtsies.formatstring import linesplit
 from curtsies.fmtfuncs import blue, red, green
 
-class Connection(object):
+class Connection:
     def __init__(self, sock):
         self.sock = sock
         self.received = []
