@@ -1,4 +1,3 @@
-# -*- coding: UTF8 -*-
 import unittest
 from functools import partial
 
@@ -14,14 +13,14 @@ class TestChrByte(unittest.TestCase):
 
     def test_helpers(self):
         self.assertEqual(events.chr_byte(97), b"a")
-        self.assertEqual(events.chr_uni(97), u"a")
+        self.assertEqual(events.chr_uni(97), "a")
 
 
 class TestCurtsiesNames(unittest.TestCase):
     def spot_check(self):
-        self.assertEqual(events.CURTSIES_NAMES[b"\x1b\x08"], u"<Esc+Ctrl-H>")
-        self.assertEqual(events.CURTSIES_NAMES[b"\x00"], u"<Ctrl-SPACE>")
-        self.assertEqual(events.CURTSIES_NAMES[b"\xea"], u"<Meta-j>")
+        self.assertEqual(events.CURTSIES_NAMES[b"\x1b\x08"], "<Esc+Ctrl-H>")
+        self.assertEqual(events.CURTSIES_NAMES[b"\x00"], "<Ctrl-SPACE>")
+        self.assertEqual(events.CURTSIES_NAMES[b"\xea"], "<Meta-j>")
 
     def test_all_values_unicode(self):
         for seq, e in events.CURTSIES_NAMES.items():
@@ -29,7 +28,7 @@ class TestCurtsiesNames(unittest.TestCase):
 
     def test_all_keys_bytes(self):
         for seq, e in events.CURTSIES_NAMES.items():
-            self.assertEqual(type(e), type(u""))
+            self.assertEqual(type(e), str)
 
 
 class TestDecodable(unittest.TestCase):
@@ -43,19 +42,19 @@ class TestGetKey(unittest.TestCase):
         get_utf_full = partial(
             events.get_key, encoding="utf-8", keynames="curtsies", full=True
         )
-        self.assertEqual(get_utf_full([b"h"]), u"h")
-        self.assertEqual(get_utf_full([b"\x1b", b"["]), u"<Esc+[>")
+        self.assertEqual(get_utf_full([b"h"]), "h")
+        self.assertEqual(get_utf_full([b"\x1b", b"["]), "<Esc+[>")
         self.assertRaises(UnicodeDecodeError, get_utf_full, [b"\xfe\xfe"])
-        self.assertRaises(ValueError, get_utf_full, u"a")
+        self.assertRaises(ValueError, get_utf_full, "a")
 
     def test_utf8(self):
         get_utf = partial(
             events.get_key, encoding="utf-8", keynames="curtsies", full=False
         )
-        self.assertEqual(get_utf([b"h"]), u"h")
+        self.assertEqual(get_utf([b"h"]), "h")
         self.assertEqual(get_utf([b"\x1b", b"["]), None)
         self.assertEqual(get_utf([b"\xe2"]), None)
-        self.assertRaises(ValueError, get_utf, u"a")
+        self.assertRaises(ValueError, get_utf, "a")
 
     def test_multibyte_utf8(self):
         get_utf = partial(
@@ -63,11 +62,11 @@ class TestGetKey(unittest.TestCase):
         )
         self.assertEqual(get_utf([b"\xc3"]), None)
         self.assertEqual(get_utf([b"\xe2"]), None)
-        self.assertEqual(get_utf([b"\xe2"], full=True), u"<Meta-b>")
-        self.assertEqual(get_utf([b"\xc3", b"\x9f"]), u"ß")
+        self.assertEqual(get_utf([b"\xe2"], full=True), "<Meta-b>")
+        self.assertEqual(get_utf([b"\xc3", b"\x9f"]), "ß")
         self.assertEqual(get_utf([b"\xe2"]), None)
         self.assertEqual(get_utf([b"\xe2", b"\x88"]), None)
-        self.assertEqual(get_utf([b"\xe2", b"\x88", b"\x82"]), u"∂")
+        self.assertEqual(get_utf([b"\xe2", b"\x88", b"\x82"]), "∂")
 
     def test_sequences_without_names(self):
         get_utf = partial(
@@ -89,15 +88,15 @@ class TestGetKeyAscii(unittest.TestCase):
         get_ascii_full = partial(
             events.get_key, encoding="ascii", keynames="curtsies", full=True
         )
-        self.assertEqual(get_ascii_full([b"a"]), u"a")
-        self.assertEqual(get_ascii_full([b"\xe1"]), u"<Meta-a>")
-        self.assertEqual(get_ascii_full([b"\xe1"], keynames="curses"), u"xE1")
+        self.assertEqual(get_ascii_full([b"a"]), "a")
+        self.assertEqual(get_ascii_full([b"\xe1"]), "<Meta-a>")
+        self.assertEqual(get_ascii_full([b"\xe1"], keynames="curses"), "xE1")
 
     def test_simple(self):
         get_ascii_full = partial(events.get_key, encoding="ascii", keynames="curtsies")
-        self.assertEqual(get_ascii_full([b"a"]), u"a")
-        self.assertEqual(get_ascii_full([b"\xe1"]), u"<Meta-a>")
-        self.assertEqual(get_ascii_full([b"\xe1"], keynames="curses"), u"xE1")
+        self.assertEqual(get_ascii_full([b"a"]), "a")
+        self.assertEqual(get_ascii_full([b"\xe1"]), "<Meta-a>")
+        self.assertEqual(get_ascii_full([b"\xe1"], keynames="curses"), "xE1")
 
 
 class TestUnknownEncoding(unittest.TestCase):
@@ -106,7 +105,7 @@ class TestUnknownEncoding(unittest.TestCase):
         self.assertEqual(get_utf16([b"a"]), None)
         self.assertEqual(get_utf16([b"a"], full=True), None)
         self.assertEqual(get_utf16([b"\xe1"]), None)
-        self.assertEqual(get_utf16([b"\xe1"], full=True), u"<Meta-a>")
+        self.assertEqual(get_utf16([b"\xe1"], full=True), "<Meta-a>")
 
 
 class TestSpecialKeys(unittest.TestCase):
@@ -114,10 +113,10 @@ class TestSpecialKeys(unittest.TestCase):
         seq = [b"\x1b", b"[", b"1", b";", b"9", b"C"]
         self.assertEqual(
             [events.get_key(seq[:i], encoding="utf8") for i in range(1, len(seq) + 1)],
-            [None, None, None, None, None, u"<Esc+RIGHT>"],
+            [None, None, None, None, None, "<Esc+RIGHT>"],
         )
 
 
 class TestPPEvent(unittest.TestCase):
     def test(self):
-        self.assertEqual(events.pp_event(u"a"), "a")
+        self.assertEqual(events.pp_event("a"), "a")
